@@ -4,32 +4,34 @@ cdf_helpers.py
 High level functionality built on top of Cognite's Python SDK
 
 Most functions require the global cdf client to be set up before using them.
-
-Files handling functions work on an `mlops` folder. This is hard-coded,
-but it can be easily modified.
 """
 import time
+import sys
 import os 
 import json
 from shutil import make_archive, unpack_archive
 
 from cognite.client.exceptions import CogniteNotFoundError, CogniteAPIError
 
-from akerbp.mlops.core import logger 
+from akerbp.mlops.core import logger, config
 
-logging=logger.get_logger(name='cdf_helper')
+logging=logger.get_logger(name='mlops_cdf')
 
 global_client = {}
+api_keys=config.api_keys
+
+env_vars = {k.upper():str(v) for k,v in config.envs_dir.items() if v}
+env_vars.update({'PLATFORM':'cdf'})
+try:
+    env_vars.pop('GOOGLE_PROJECT_ID')
+except KeyError:
+    pass
 
-api_keys = dict(
-    functions = os.getenv('COGNITE_API_KEY_FUNCTIONS'),
-    data = os.getenv('COGNITE_API_KEY_DATA'),
-    files = os.getenv('COGNITE_API_KEY_FILES')
-)
 
 def set_up_cdf_client(context='run'):
     """
-    Set up the global client used by most helpers. This needs to be called before using any helper. Should be called once with all required keys.
+    Set up the global client used by most helpers. This needs to be called
+    before using any helper. 
 
     Input:
       - context: string either 'run' (access to data and functions) or 'deploy'
@@ -43,7 +45,8 @@ def set_up_cdf_client(context='run'):
         raise ValueError("Context should be either 'run' or 'deploy'")
 
     for k in api_key_labels:
-        global_client[k] = get_client(api_keys[k], k)
+        if k not in global_client:
+            global_client[k] = get_client(api_keys[k], k)
         
     logging.info("CDF client was set up correctly")
 
@@ -66,8 +69,6 @@ def get_client(api_key, api_key_label=None):
     )
     assert client.login.status().logged_in
     logging.debug(f"{client.version=}")
-    logging.debug(f"{client.config.client_name=}")
-    
     return client
 
 
@@ -95,6 +96,7 @@ def create_function(
         logging.debug(f"Deleted function {function_name}")
     except CogniteNotFoundError:
         pass
+    
     function = client.functions.create( 
         name=function_name, 
         folder=folder, 
@@ -102,9 +104,12 @@ def create_function(
         external_id=function_name,
         description=description,
         owner=owner,
-        secrets=secrets
+        secrets=secrets,
+        env_vars=env_vars
+    )
+    logging.debug(
+        f"Created {function_name}: {folder=}, {handler_path=}, {env_vars=}"
     )
-    logging.debug(f"Created {function_name}: {folder=}, {handler_path=}")
 
     return function
 
@@ -143,6 +148,8 @@ def deploy_function(
         if function.status == 'Ready':
             break
         if function.status == 'Failed' and trial < max_errors-1:
+            logging.warning(f"Function failed: {function.id=}")
+            logging.debug(f"Error was: {function.error=}")
             logging.debug(f"Try to create function again")
         else:
             raise Exception(f"Function deployment error: {function.error=}")
@@ -221,7 +228,7 @@ def upload_file(
     external_id, 
     path, 
     metadata={}, 
-    directory='/mlops',
+    directory='/',
     overwrite=True):
     """
     Upload file to Cognite
@@ -249,7 +256,13 @@ def upload_file(
     return file_info
 
 
-def upload_folder(external_id, path, metadata={}, overwrite=False):
+def upload_folder(
+    external_id, 
+    path, 
+    metadata={}, 
+    overwrite=False, 
+    target_folder='/'
+):
     """
     Upload folder content to Cognite. It compresses the folder and uploads it.
     
@@ -257,6 +270,7 @@ def upload_folder(external_id, path, metadata={}, overwrite=False):
         - external_id: external id (should be unique in the CDF project)
         - path: path of local folder where content is stored
         - metadata: dictionary with file metadata
+        - target_folder: path where compressed file should be stored
         - overwrite: if overwrite==False and `external_id` exists => exception
     """
     base_name = os.path.join(path, 'archive')
@@ -265,7 +279,8 @@ def upload_folder(external_id, path, metadata={}, overwrite=False):
         external_id, 
         archive_name, 
         metadata=metadata,
-        overwrite=overwrite
+        overwrite=overwrite,
+        directory=target_folder
     )
     os.remove(archive_name)
     logging.info(f"Folder content uploaded: {file_info=}")
@@ -293,14 +308,16 @@ def log_system_info():
     """
     Can be called from a handler to log CDF environment information
     """
-    logging.debug(os.popen('python --version').read())
-    logging.debug(os.popen('ls -la *').read())
-    logging.debug(os.popen('pip freeze').read())
+    logging.debug(f"Python version:\n{os.popen('python --version').read()}")
+    logging.debug(f"Python path:\n{sys.path}")
+    logging.debug(f"Current working directory:\n{os.getcwd()}")
+    logging.debug(f"Content:\n{os.popen('ls -la *').read()}")
+    logging.debug(f"Packages:\n{os.popen('pip freeze').read()}")
 
 
-def count_file_versions(
+def count_files(
     external_id_prefix, 
-    directory_prefix='/mlops'
+    directory_prefix='/'
 ):
     """
     How many files have the given `external_id_prefix` and `directory_prefix`
@@ -316,83 +333,28 @@ def count_file_versions(
     return result[0]["count"]
 
 
-def upload_new_model_version(model_name, env, folder, metadata):
+def query_file_versions(
+    directory_prefix, 
+    external_id_prefix, 
+    metadata={}, 
+    uploaded=True):
     """
-    Upload a new model version. Files in a folder are archived and stored 
-    with external id `model_name/env/version`, where version is automatically increased. 
+    Find all file versions that match a query. 
 
     Input:
-        -model_name: name of the model 
-        -env: name of the environment ('dev', 'test', 'prod')
-        -folder: path to folder whose content will be uploaded
-        -metadata: dictionary with metadata (it should not contain a 'version'
-        key)
-    
+        -directory_prefix
+        -external_id_prefix
+        -metadata: query to the metadata (dictionary)
     Output:
-        - model metadata (dictionary)
-    """
-
-    filter = f"{model_name}/{env}/"
-    version = count_file_versions(filter) + 1
-    metadata["version"] = version
-    external_id = f"{model_name}/{env}/{version}"
-
-    folder_info = upload_folder(
-        external_id, 
-        folder, 
-        metadata
-    )
-    logging.info(f"Uploaded model with {external_id=} from {folder}")
-    return folder_info
-
-
-def find_model_version(model_name, env, metadata):
-    """
-    Model external id is specified by the model name and the environment
-    (starts with `{model_name}/{env}`), and a query to the metadata. If this is
-    not enough, the latest version is chosen.
-
-    Input:
-        -model_name: name of the model 
-        -env: name of the environment ('dev', 'test', 'prod')
-        -metadata: query to the metadata (dictionary), it can contain a
-        'version' key
+        - list of versions (dataframe)
     """
     client = global_client["files"]
     file_list = client.files.list(
         limit=-1, 
-        directory_prefix='/mlops', 
-        external_id_prefix=f"{model_name}/{env}",
+        directory_prefix=directory_prefix, 
+        external_id_prefix=external_id_prefix,
         metadata=metadata,
-        uploaded=True
+        uploaded=uploaded
     ).to_pandas()
 
-    if (n_models := file_list.shape[0]) == 0:
-        message = f"No model found with {model_name=} in {env=} and {metadata}"
-        raise Exception(message)
-    elif n_models > 1:
-        logging.debug(f"Found {n_models} model files. Will choose the latest.")
-    
-    # Get latest in case there are more than one
-    external_id = file_list.loc[file_list.uploadedTime.argmax(), "externalId"]
-    return external_id
-
-
-def download_model_version(model_name, env, folder, metadata={}):
-    """
-    Download a model version to a folder. First the model's external id is
-    found, and then it is downloaded to the chosen folder (creating the folder
-    if necessary).
-
-    Input: 
-        -model_name: name of the model 
-        -env: name of the environment ('dev', 'test', 'prod')
-        -folder: path to folder where the content will be uploaded
-        -metadata: query to the metadata (dictionary)
-    """
-    external_id = find_model_version(model_name, env, metadata)
-    if not os.path.isdir(folder):
-        os.mkdir(folder)
-    download_folder(external_id, folder)
-    logging.info(f"Downloaded model with {external_id=} to {folder}")
-    return external_id
\ No newline at end of file
+    return file_list
